@@ -1,4 +1,4 @@
-/* -*- mode: C; c-file-style: "gnu" -*- */
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /* config-parser.c  XML-library-agnostic configuration file parser
  *
  * Copyright (C) 2003, 2004 Red Hat, Inc.
@@ -17,9 +17,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
+
+#include <config.h>
+#include "config-parser-common.h"
 #include "config-parser.h"
 #include "test.h"
 #include "utils.h"
@@ -27,30 +30,9 @@
 #include "selinux.h"
 #include <dbus/dbus-list.h>
 #include <dbus/dbus-internals.h>
+#include <dbus/dbus-sysdeps.h>
 #include <string.h>
 
-typedef enum
-{
-  ELEMENT_NONE,
-  ELEMENT_BUSCONFIG,
-  ELEMENT_INCLUDE,
-  ELEMENT_USER,
-  ELEMENT_LISTEN,
-  ELEMENT_AUTH,
-  ELEMENT_POLICY,
-  ELEMENT_LIMIT,
-  ELEMENT_ALLOW,
-  ELEMENT_DENY,
-  ELEMENT_FORK,
-  ELEMENT_PIDFILE,
-  ELEMENT_SERVICEDIR,
-  ELEMENT_INCLUDEDIR,
-  ELEMENT_TYPE,
-  ELEMENT_SELINUX,
-  ELEMENT_ASSOCIATE,
-  ELEMENT_STANDARD_SESSION_SERVICEDIRS
-} ElementType;
-
 typedef enum
 {
   /* we ignore policies for unknown groups/users */
@@ -108,13 +90,15 @@ struct BusConfigParser
 
   char *user;          /**< user to run as */
 
+  char *servicehelper; /**< location of the setuid helper */
+
   char *bus_type;          /**< Message bus type */
   
   DBusList *listen_on; /**< List of addresses to listen to */
 
   DBusList *mechanisms; /**< Auth mechanisms */
 
-  DBusList *service_dirs; /**< Directories to look for services in */
+  DBusList *service_dirs; /**< Directories to look for session services in */
 
   DBusList *conf_dirs;   /**< Directories to look for policy configuration in */
 
@@ -130,57 +114,14 @@ struct BusConfigParser
 
   unsigned int fork : 1; /**< TRUE to fork into daemon mode */
 
+  unsigned int syslog : 1; /**< TRUE to enable syslog */
+  unsigned int keep_umask : 1; /**< TRUE to keep original umask when forking */
+
   unsigned int is_toplevel : 1; /**< FALSE if we are a sub-config-file inside another one */
+
+  unsigned int allow_anonymous : 1; /**< TRUE to allow anonymous connections */
 };
 
-static const char*
-element_type_to_name (ElementType type)
-{
-  switch (type)
-    {
-    case ELEMENT_NONE:
-      return NULL;
-    case ELEMENT_BUSCONFIG:
-      return "busconfig";
-    case ELEMENT_INCLUDE:
-      return "include";
-    case ELEMENT_USER:
-      return "user";
-    case ELEMENT_LISTEN:
-      return "listen";
-    case ELEMENT_AUTH:
-      return "auth";
-    case ELEMENT_POLICY:
-      return "policy";
-    case ELEMENT_LIMIT:
-      return "limit";
-    case ELEMENT_ALLOW:
-      return "allow";
-    case ELEMENT_DENY:
-      return "deny";
-    case ELEMENT_FORK:
-      return "fork";
-    case ELEMENT_PIDFILE:
-      return "pidfile";
-    case ELEMENT_STANDARD_SESSION_SERVICEDIRS:
-      return "standard_session_servicedirs";
-    case ELEMENT_SERVICEDIR:
-      return "servicedir";
-    case ELEMENT_INCLUDEDIR:
-      return "includedir";
-    case ELEMENT_TYPE:
-      return "type";
-    case ELEMENT_SELINUX:
-      return "selinux";
-    case ELEMENT_ASSOCIATE:
-      return "associate";
-    }
-
-  _dbus_assert_not_reached ("bad element type");
-
-  return NULL;
-}
-
 static Element*
 push_element (BusConfigParser *parser,
               ElementType      type)
@@ -373,6 +314,9 @@ merge_included (BusConfigParser *parser,
   if (included->fork)
     parser->fork = TRUE;
 
+  if (included->keep_umask)
+    parser->keep_umask = TRUE;
+
   if (included->pidfile != NULL)
     {
       dbus_free (parser->pidfile);
@@ -463,6 +407,15 @@ bus_config_parser_new (const DBusString      *basedir,
       parser->limits.max_incoming_bytes = _DBUS_ONE_MEGABYTE * 127;
       parser->limits.max_outgoing_bytes = _DBUS_ONE_MEGABYTE * 127;
       parser->limits.max_message_size = _DBUS_ONE_MEGABYTE * 32;
+
+      /* We set relatively conservative values here since due to the
+      way SCM_RIGHTS works we need to preallocate an array for the
+      maximum number of file descriptors we can receive. Picking a
+      high value here thus translates directly to more memory
+      allocation. */
+      parser->limits.max_incoming_unix_fds = 1024*4;
+      parser->limits.max_outgoing_unix_fds = 1024*4;
+      parser->limits.max_message_unix_fds = 1024;
       
       /* Making this long means the user has to wait longer for an error
        * message if something screws up, but making it too short means
@@ -487,10 +440,18 @@ bus_config_parser_new (const DBusString      *basedir,
       
       parser->limits.max_pending_activations = 512;
       parser->limits.max_services_per_connection = 512;
-      
+
+      /* For this one, keep in mind that it isn't only the memory used
+       * by the match rules, but slowdown from linearly walking a big
+       * list of them. A client adding more than this is almost
+       * certainly a bad idea for that reason, and should change to a
+       * smaller number of wider-net match rules - getting every last
+       * message to the bus is probably better than having a thousand
+       * match rules.
+       */
       parser->limits.max_match_rules_per_connection = 512;
       
-      parser->limits.reply_timeout = 5 * 60 * 1000; /* 5 minutes */
+      parser->limits.reply_timeout = -1; /* never */
 
       /* this is effectively a limit on message queue size for messages
        * that require a reply
@@ -526,6 +487,7 @@ bus_config_parser_unref (BusConfigParser *parser)
         pop_element (parser);
 
       dbus_free (parser->user);
+      dbus_free (parser->servicehelper);
       dbus_free (parser->bus_type);
       dbus_free (parser->pidfile);
       
@@ -640,9 +602,6 @@ locate_attributes (BusConfigParser  *parser,
 
   va_end (args);
 
-  if (!retval)
-    return retval;
-
   i = 0;
   while (attribute_names[i])
     {
@@ -714,7 +673,11 @@ start_busconfig_child (BusConfigParser   *parser,
                        const char       **attribute_values,
                        DBusError         *error)
 {
-  if (strcmp (element_name, "user") == 0)
+  ElementType element_type;
+
+  element_type = bus_config_parser_element_name_to_type (element_name);
+
+  if (element_type == ELEMENT_USER)
     {
       if (!check_no_attributes (parser, "user", attribute_names, attribute_values, error))
         return FALSE;
@@ -727,12 +690,12 @@ start_busconfig_child (BusConfigParser   *parser,
 
       return TRUE;
     }
-  else if (strcmp (element_name, "type") == 0)
+  else if (element_type == ELEMENT_CONFIGTYPE)
     {
       if (!check_no_attributes (parser, "type", attribute_names, attribute_values, error))
         return FALSE;
 
-      if (push_element (parser, ELEMENT_TYPE) == NULL)
+      if (push_element (parser, ELEMENT_CONFIGTYPE) == NULL)
         {
           BUS_SET_OOM (error);
           return FALSE;
@@ -740,7 +703,7 @@ start_busconfig_child (BusConfigParser   *parser,
 
       return TRUE;
     }
-  else if (strcmp (element_name, "fork") == 0)
+  else if (element_type == ELEMENT_FORK)
     {
       if (!check_no_attributes (parser, "fork", attribute_names, attribute_values, error))
         return FALSE;
@@ -755,7 +718,37 @@ start_busconfig_child (BusConfigParser   *parser,
       
       return TRUE;
     }
-  else if (strcmp (element_name, "pidfile") == 0)
+  else if (element_type == ELEMENT_SYSLOG)
+    {
+      if (!check_no_attributes (parser, "syslog", attribute_names, attribute_values, error))
+        return FALSE;
+
+      if (push_element (parser, ELEMENT_SYSLOG) == NULL)
+        {
+          BUS_SET_OOM (error);
+          return FALSE;
+        }
+      
+      parser->syslog = TRUE;
+      
+      return TRUE;
+    }
+  else if (element_type == ELEMENT_KEEP_UMASK)
+    {
+      if (!check_no_attributes (parser, "keep_umask", attribute_names, attribute_values, error))
+        return FALSE;
+
+      if (push_element (parser, ELEMENT_KEEP_UMASK) == NULL)
+        {
+          BUS_SET_OOM (error);
+          return FALSE;
+        }
+
+      parser->keep_umask = TRUE;
+      
+      return TRUE;
+    }
+  else if (element_type == ELEMENT_PIDFILE)
     {
       if (!check_no_attributes (parser, "pidfile", attribute_names, attribute_values, error))
         return FALSE;
@@ -768,7 +761,7 @@ start_busconfig_child (BusConfigParser   *parser,
 
       return TRUE;
     }
-  else if (strcmp (element_name, "listen") == 0)
+  else if (element_type == ELEMENT_LISTEN)
     {
       if (!check_no_attributes (parser, "listen", attribute_names, attribute_values, error))
         return FALSE;
@@ -781,7 +774,7 @@ start_busconfig_child (BusConfigParser   *parser,
 
       return TRUE;
     }
-  else if (strcmp (element_name, "auth") == 0)
+  else if (element_type == ELEMENT_AUTH)
     {
       if (!check_no_attributes (parser, "auth", attribute_names, attribute_values, error))
         return FALSE;
@@ -794,7 +787,20 @@ start_busconfig_child (BusConfigParser   *parser,
       
       return TRUE;
     }
-  else if (strcmp (element_name, "includedir") == 0)
+  else if (element_type == ELEMENT_SERVICEHELPER)
+    {
+      if (!check_no_attributes (parser, "servicehelper", attribute_names, attribute_values, error))
+        return FALSE;
+
+      if (push_element (parser, ELEMENT_SERVICEHELPER) == NULL)
+        {
+          BUS_SET_OOM (error);
+          return FALSE;
+        }
+
+      return TRUE;
+    }
+  else if (element_type == ELEMENT_INCLUDEDIR)
     {
       if (!check_no_attributes (parser, "includedir", attribute_names, attribute_values, error))
         return FALSE;
@@ -807,7 +813,7 @@ start_busconfig_child (BusConfigParser   *parser,
 
       return TRUE;
     }
-  else if (strcmp (element_name, "standard_session_servicedirs") == 0)
+  else if (element_type == ELEMENT_STANDARD_SESSION_SERVICEDIRS)
     {
       DBusList *link;
       DBusList *dirs;
@@ -833,7 +839,47 @@ start_busconfig_child (BusConfigParser   *parser,
 
       return TRUE;
     }
-  else if (strcmp (element_name, "servicedir") == 0)
+  else if (element_type == ELEMENT_STANDARD_SYSTEM_SERVICEDIRS)
+    {
+      DBusList *link;
+      DBusList *dirs;
+      dirs = NULL;
+
+      if (!check_no_attributes (parser, "standard_system_servicedirs", attribute_names, attribute_values, error))
+        return FALSE;
+
+      if (push_element (parser, ELEMENT_STANDARD_SYSTEM_SERVICEDIRS) == NULL)
+        {
+          BUS_SET_OOM (error);
+          return FALSE;
+        }
+
+      if (!_dbus_get_standard_system_servicedirs (&dirs))
+        {
+          BUS_SET_OOM (error);
+          return FALSE;
+        }
+
+        while ((link = _dbus_list_pop_first_link (&dirs)))
+          service_dirs_append_link_unique_or_free (&parser->service_dirs, link);
+
+      return TRUE;
+    }
+  else if (element_type == ELEMENT_ALLOW_ANONYMOUS)
+    {
+      if (!check_no_attributes (parser, "allow_anonymous", attribute_names, attribute_values, error))
+        return FALSE;
+
+      if (push_element (parser, ELEMENT_ALLOW_ANONYMOUS) == NULL)
+        {
+          BUS_SET_OOM (error);
+          return FALSE;
+        }
+
+      parser->allow_anonymous = TRUE;
+      return TRUE;
+    }
+  else if (element_type == ELEMENT_SERVICEDIR)
     {
       if (!check_no_attributes (parser, "servicedir", attribute_names, attribute_values, error))
         return FALSE;
@@ -846,7 +892,7 @@ start_busconfig_child (BusConfigParser   *parser,
 
       return TRUE;
     }
-  else if (strcmp (element_name, "include") == 0)
+  else if (element_type == ELEMENT_INCLUDE)
     {
       Element *e;
       const char *if_selinux_enabled;
@@ -919,7 +965,7 @@ start_busconfig_child (BusConfigParser   *parser,
 
       return TRUE;
     }
-  else if (strcmp (element_name, "policy") == 0)
+  else if (element_type == ELEMENT_POLICY)
     {
       Element *e;
       const char *context;
@@ -1026,7 +1072,7 @@ start_busconfig_child (BusConfigParser   *parser,
       
       return TRUE;
     }
-  else if (strcmp (element_name, "limit") == 0)
+  else if (element_type == ELEMENT_LIMIT)
     {
       Element *e;
       const char *name;
@@ -1061,7 +1107,7 @@ start_busconfig_child (BusConfigParser   *parser,
 
       return TRUE;
     }
-  else if (strcmp (element_name, "selinux") == 0)
+  else if (element_type == ELEMENT_SELINUX)
     {
       if (!check_no_attributes (parser, "selinux", attribute_names, attribute_values, error))
         return FALSE;
@@ -1091,6 +1137,7 @@ append_rule_from_element (BusConfigParser   *parser,
                           dbus_bool_t        allow,
                           DBusError         *error)
 {
+  const char *log;
   const char *send_interface;
   const char *send_member;
   const char *send_error;
@@ -1134,6 +1181,7 @@ append_rule_from_element (BusConfigParser   *parser,
                           "own", &own,
                           "user", &user,
                           "group", &group,
+                          "log", &log,
                           NULL))
     return FALSE;
 
@@ -1175,97 +1223,96 @@ append_rule_from_element (BusConfigParser   *parser,
    * Pretty sure the below stuff is broken, FIXME think about it more.
    */
 
-  if (((send_interface && send_error) ||
-       (send_interface && receive_interface) ||
-       (send_interface && receive_member) ||
-       (send_interface && receive_error) ||
-       (send_interface && receive_sender) ||
-       (send_interface && receive_requested_reply) ||
-       (send_interface && own) ||
-       (send_interface && user) ||
-       (send_interface && group)) ||
+  if ((send_interface && (send_error ||
+                          receive_interface ||
+                          receive_member ||
+                          receive_error ||
+                          receive_sender ||
+                          receive_requested_reply ||
+                          own ||
+                          user ||
+                          group)) ||
 
-      ((send_member && send_error) ||
-       (send_member && receive_interface) ||
-       (send_member && receive_member) ||
-       (send_member && receive_error) ||
-       (send_member && receive_sender) ||
-       (send_member && receive_requested_reply) ||
-       (send_member && own) ||
-       (send_member && user) ||
-       (send_member && group)) ||
-      
-      ((send_error && receive_interface) ||
-       (send_error && receive_member) ||
-       (send_error && receive_error) ||
-       (send_error && receive_sender) ||
-       (send_error && receive_requested_reply) ||
-       (send_error && own) ||
-       (send_error && user) ||
-       (send_error && group)) ||
+      (send_member && (send_error ||
+                       receive_interface ||
+                       receive_member ||
+                       receive_error ||
+                       receive_sender ||
+                       receive_requested_reply ||
+                       own ||
+                       user ||
+                       group)) ||
 
-      ((send_destination && receive_interface) ||
-       (send_destination && receive_member) ||
-       (send_destination && receive_error) ||
-       (send_destination && receive_sender) ||
-       (send_destination && receive_requested_reply) ||
-       (send_destination && own) ||
-       (send_destination && user) ||
-       (send_destination && group)) ||
+      (send_error && (receive_interface ||
+                      receive_member ||
+                      receive_error ||
+                      receive_sender ||
+                      receive_requested_reply ||
+                      own ||
+                      user ||
+                      group)) ||
 
-      ((send_type && receive_interface) ||
-       (send_type && receive_member) ||
-       (send_type && receive_error) ||
-       (send_type && receive_sender) ||
-       (send_type && receive_requested_reply) ||
-       (send_type && own) ||
-       (send_type && user) ||
-       (send_type && group)) ||
+      (send_destination && (receive_interface ||
+                            receive_member ||
+                            receive_error ||
+                            receive_sender ||
+                            receive_requested_reply ||
+                            own ||
+                            user ||
+                            group)) ||
 
-      ((send_path && receive_interface) ||
-       (send_path && receive_member) ||
-       (send_path && receive_error) ||
-       (send_path && receive_sender) ||
-       (send_path && receive_requested_reply) ||
-       (send_path && own) ||
-       (send_path && user) ||
-       (send_path && group)) ||
+      (send_type && (receive_interface ||
+                     receive_member ||
+                     receive_error ||
+                     receive_sender ||
+                     receive_requested_reply ||
+                     own ||
+                     user ||
+                     group)) ||
 
-      ((send_requested_reply && receive_interface) ||
-       (send_requested_reply && receive_member) ||
-       (send_requested_reply && receive_error) ||
-       (send_requested_reply && receive_sender) ||
-       (send_requested_reply && receive_requested_reply) ||
-       (send_requested_reply && own) ||
-       (send_requested_reply && user) ||
-       (send_requested_reply && group)) ||
-      
-      ((receive_interface && receive_error) ||
-       (receive_interface && own) ||
-       (receive_interface && user) ||
-       (receive_interface && group)) ||
+      (send_path && (receive_interface ||
+                     receive_member ||
+                     receive_error ||
+                     receive_sender ||
+                     receive_requested_reply ||
+                     own ||
+                     user ||
+                     group)) ||
 
-      ((receive_member && receive_error) ||
-       (receive_member && own) ||
-       (receive_member && user) ||
-       (receive_member && group)) ||
-      
-      ((receive_error && own) ||
-       (receive_error && user) ||
-       (receive_error && group)) ||
+      (send_requested_reply && (receive_interface ||
+                                receive_member ||
+                                receive_error ||
+                                receive_sender ||
+                                receive_requested_reply ||
+                                own ||
+                                user ||
+                                group)) ||
 
-      ((eavesdrop && own) ||
-       (eavesdrop && user) ||
-       (eavesdrop && group)) ||
+      (receive_interface && (receive_error ||
+                             own ||
+                             user ||
+                             group)) ||
 
-      ((receive_requested_reply && own) ||
-       (receive_requested_reply && user) ||
-       (receive_requested_reply && group)) ||
-      
-      ((own && user) ||
-       (own && group)) ||
+      (receive_member && (receive_error ||
+                          own ||
+                          user ||
+                          group)) ||
 
-      ((user && group)))
+      (receive_error && (own ||
+                         user ||
+                         group)) ||
+
+      (eavesdrop && (own ||
+                     user ||
+                     group)) ||
+
+      (receive_requested_reply && (own ||
+                                   user ||
+                                   group)) ||
+
+      (own && (user || group)) ||
+
+      (user && group))
     {
       dbus_set_error (error, DBUS_ERROR_FAILED,
                       "Invalid combination of attributes on element <%s>",
@@ -1338,6 +1385,9 @@ append_rule_from_element (BusConfigParser   *parser,
       if (eavesdrop)
         rule->d.send.eavesdrop = (strcmp (eavesdrop, "true") == 0);
 
+      if (log)
+        rule->d.send.log = (strcmp (log, "true") == 0);
+
       if (send_requested_reply)
         rule->d.send.requested_reply = (strcmp (send_requested_reply, "true") == 0);
 
@@ -1789,16 +1839,31 @@ set_limit (BusConfigParser *parser,
       must_be_positive = TRUE;
       parser->limits.max_incoming_bytes = value;
     }
+  else if (strcmp (name, "max_incoming_unix_fds") == 0)
+    {
+      must_be_positive = TRUE;
+      parser->limits.max_incoming_unix_fds = value;
+    }
   else if (strcmp (name, "max_outgoing_bytes") == 0)
     {
       must_be_positive = TRUE;
       parser->limits.max_outgoing_bytes = value;
     }
+  else if (strcmp (name, "max_outgoing_unix_fds") == 0)
+    {
+      must_be_positive = TRUE;
+      parser->limits.max_outgoing_unix_fds = value;
+    }
   else if (strcmp (name, "max_message_size") == 0)
     {
       must_be_positive = TRUE;
       parser->limits.max_message_size = value;
     }
+  else if (strcmp (name, "max_message_unix_fds") == 0)
+    {
+      must_be_positive = TRUE;
+      parser->limits.max_message_unix_fds = value;
+    }
   else if (strcmp (name, "service_start_timeout") == 0)
     {
       must_be_positive = TRUE;
@@ -1912,7 +1977,7 @@ bus_config_parser_end_element (BusConfigParser   *parser,
       return FALSE;
     }
 
-  n = element_type_to_name (t);
+  n = bus_config_parser_element_type_to_name (t);
   _dbus_assert (n != NULL);
   if (strcmp (n, element_name) != 0)
     {
@@ -1936,18 +2001,19 @@ bus_config_parser_end_element (BusConfigParser   *parser,
 
     case ELEMENT_INCLUDE:
     case ELEMENT_USER:
-    case ELEMENT_TYPE:
+    case ELEMENT_CONFIGTYPE:
     case ELEMENT_LISTEN:
     case ELEMENT_PIDFILE:
     case ELEMENT_AUTH:
     case ELEMENT_SERVICEDIR:
+    case ELEMENT_SERVICEHELPER:
     case ELEMENT_INCLUDEDIR:
     case ELEMENT_LIMIT:
       if (!e->had_content)
         {
           dbus_set_error (error, DBUS_ERROR_FAILED,
                           "XML element <%s> was expected to have content inside it",
-                          element_type_to_name (e->type));
+                          bus_config_parser_element_type_to_name (e->type));
           return FALSE;
         }
 
@@ -1964,9 +2030,13 @@ bus_config_parser_end_element (BusConfigParser   *parser,
     case ELEMENT_ALLOW:
     case ELEMENT_DENY:
     case ELEMENT_FORK:
+    case ELEMENT_SYSLOG:
+    case ELEMENT_KEEP_UMASK:
     case ELEMENT_SELINUX:
     case ELEMENT_ASSOCIATE:
     case ELEMENT_STANDARD_SESSION_SERVICEDIRS:
+    case ELEMENT_STANDARD_SYSTEM_SERVICEDIRS:
+    case ELEMENT_ALLOW_ANONYMOUS:
       break;
     }
 
@@ -2079,6 +2149,40 @@ include_file (BusConfigParser   *parser,
     }
 }
 
+static dbus_bool_t
+servicehelper_path (BusConfigParser   *parser,
+                    const DBusString  *filename,
+                    DBusError         *error)
+{
+  const char *filename_str;
+  char *servicehelper;
+
+  filename_str = _dbus_string_get_const_data (filename);
+
+  /* copy to avoid overwriting with NULL on OOM */
+  servicehelper = _dbus_strdup (filename_str);
+
+  /* check for OOM */
+  if (servicehelper == NULL)
+    {
+      BUS_SET_OOM (error);
+      return FALSE;
+    }
+
+  /* save the latest servicehelper only if not OOM */
+  dbus_free (parser->servicehelper);
+  parser->servicehelper = servicehelper;
+
+  /* We don't check whether the helper exists; instead we
+   * would just fail to ever activate anything if it doesn't.
+   * This allows an admin to fix the problem if it doesn't exist.
+   * It also allows the parser test suite to successfully parse
+   * test cases without installing the helper. ;-)
+   */
+  
+  return TRUE;
+}
+
 static dbus_bool_t
 include_dir (BusConfigParser   *parser,
              const DBusString  *dirname,
@@ -2132,8 +2236,19 @@ include_dir (BusConfigParser   *parser,
         {
           if (!include_file (parser, &full_path, TRUE, error))
             {
-              _dbus_string_free (&full_path);
-              goto failed;
+              if (dbus_error_is_set (error))
+                {
+                  /* We log to syslog unconditionally here, because this is
+                   * the configuration parser, so we don't yet know whether
+                   * this bus is going to want to write to syslog! (There's
+                   * also some layer inversion going on, if we want to use
+                   * the bus context.) */
+                  _dbus_system_log (DBUS_SYSTEM_LOG_INFO,
+                                    "Encountered error '%s' while parsing '%s'\n",
+                                    error->message,
+                                    _dbus_string_get_const_data (&full_path));
+                  dbus_error_free (error);
+                }
             }
         }
 
@@ -2214,7 +2329,11 @@ bus_config_parser_content (BusConfigParser   *parser,
     case ELEMENT_ALLOW:
     case ELEMENT_DENY:
     case ELEMENT_FORK:
+    case ELEMENT_SYSLOG:
+    case ELEMENT_KEEP_UMASK:
     case ELEMENT_STANDARD_SESSION_SERVICEDIRS:    
+    case ELEMENT_STANDARD_SYSTEM_SERVICEDIRS:    
+    case ELEMENT_ALLOW_ANONYMOUS:
     case ELEMENT_SELINUX:
     case ELEMENT_ASSOCIATE:
       if (all_whitespace (content))
@@ -2223,7 +2342,7 @@ bus_config_parser_content (BusConfigParser   *parser,
         {
           dbus_set_error (error, DBUS_ERROR_FAILED,
                           "No text content expected inside XML element %s in configuration file",
-                          element_type_to_name (top_element_type (parser)));
+                          bus_config_parser_element_type_to_name (top_element_type (parser)));
           return FALSE;
         }
 
@@ -2288,6 +2407,31 @@ bus_config_parser_content (BusConfigParser   *parser,
       }
       break;
 
+    case ELEMENT_SERVICEHELPER:
+      {
+        DBusString full_path;
+        
+        e->had_content = TRUE;
+
+        if (!_dbus_string_init (&full_path))
+          goto nomem;
+        
+        if (!make_full_path (&parser->basedir, content, &full_path))
+          {
+            _dbus_string_free (&full_path);
+            goto nomem;
+          }
+
+        if (!servicehelper_path (parser, &full_path, error))
+          {
+            _dbus_string_free (&full_path);
+            return FALSE;
+          }
+
+        _dbus_string_free (&full_path);
+      }
+      break;
+      
     case ELEMENT_INCLUDEDIR:
       {
         DBusString full_path;
@@ -2327,7 +2471,7 @@ bus_config_parser_content (BusConfigParser   *parser,
       }
       break;
 
-    case ELEMENT_TYPE:
+    case ELEMENT_CONFIGTYPE:
       {
         char *s;
 
@@ -2399,6 +2543,7 @@ bus_config_parser_content (BusConfigParser   *parser,
             goto nomem;
           }
 
+        /* _only_ extra session directories can be specified */
         if (!service_dirs_append_unique_or_free (&parser->service_dirs, s))
           {
             _dbus_string_free (&full_path);
@@ -2452,7 +2597,7 @@ bus_config_parser_finished (BusConfigParser   *parser,
     {
       dbus_set_error (error, DBUS_ERROR_FAILED,
                       "Element <%s> was not closed in configuration file",
-                      element_type_to_name (top_element_type (parser)));
+                      bus_config_parser_element_type_to_name (top_element_type (parser)));
 
       return FALSE;
     }
@@ -2509,12 +2654,36 @@ bus_config_parser_get_fork (BusConfigParser   *parser)
   return parser->fork;
 }
 
+dbus_bool_t
+bus_config_parser_get_syslog (BusConfigParser   *parser)
+{
+  return parser->syslog;
+}
+
+dbus_bool_t
+bus_config_parser_get_keep_umask (BusConfigParser   *parser)
+{
+  return parser->keep_umask;
+}
+
+dbus_bool_t
+bus_config_parser_get_allow_anonymous (BusConfigParser   *parser)
+{
+  return parser->allow_anonymous;
+}
+
 const char *
 bus_config_parser_get_pidfile (BusConfigParser   *parser)
 {
   return parser->pidfile;
 }
 
+const char *
+bus_config_parser_get_servicehelper (BusConfigParser   *parser)
+{
+  return parser->servicehelper;
+}
+
 BusPolicy*
 bus_config_parser_steal_policy (BusConfigParser *parser)
 {
@@ -2847,8 +3016,11 @@ limits_equal (const BusLimits *a,
 {
   return
     (a->max_incoming_bytes == b->max_incoming_bytes
+     || a->max_incoming_unix_fds == b->max_incoming_unix_fds
      || a->max_outgoing_bytes == b->max_outgoing_bytes
+     || a->max_outgoing_unix_fds == b->max_outgoing_unix_fds
      || a->max_message_size == b->max_message_size
+     || a->max_message_unix_fds == b->max_message_unix_fds
      || a->activation_timeout == b->activation_timeout
      || a->auth_timeout == b->auth_timeout
      || a->max_completed_connections == b->max_completed_connections
@@ -2896,6 +3068,9 @@ config_parsers_equal (const BusConfigParser *a,
   if (! bools_equal (a->fork, b->fork))
     return FALSE;
 
+  if (! bools_equal (a->keep_umask, b->keep_umask))
+    return FALSE;
+
   if (! bools_equal (a->is_toplevel, b->is_toplevel))
     return FALSE;
 
@@ -3074,15 +3249,18 @@ process_test_equiv_subdir (const DBusString *test_base_dir,
   
 }
 
-static const char *test_service_dir_matches[] = 
+static const char *test_session_service_dir_matches[] = 
         {
 #ifdef DBUS_UNIX
+         "/testhome/foo/.testlocal/testshare/dbus-1/services",
          "/testusr/testlocal/testshare/dbus-1/services",
          "/testusr/testshare/dbus-1/services",
-#endif
          DBUS_DATADIR"/dbus-1/services",
-#ifdef DBUS_UNIX
-         "/testhome/foo/.testlocal/testshare/dbus-1/services",
+#endif
+/* will be filled in test_default_session_servicedirs() */
+#ifdef DBUS_WIN
+         NULL,
+         NULL,
 #endif
          NULL
         };
@@ -3093,16 +3271,28 @@ test_default_session_servicedirs (void)
   DBusList *dirs;
   DBusList *link;
   DBusString progs;
-  const char *common_progs;
   int i;
 
+#ifdef DBUS_WIN
+  const char *common_progs;
+  char buffer[1024];
+
+  if (_dbus_get_install_root(buffer, sizeof(buffer)))
+    {
+      strcat(buffer,DBUS_DATADIR);
+      strcat(buffer,"/dbus-1/services");
+      test_session_service_dir_matches[0] = buffer;
+    }
+#endif
+
   /* On Unix we don't actually use this variable, but it's easier to handle the
    * deallocation if we always allocate it, whether needed or not */
   if (!_dbus_string_init (&progs))
     _dbus_assert_not_reached ("OOM allocating progs");
 
-  common_progs = _dbus_getenv ("CommonProgramFiles");
 #ifndef DBUS_UNIX
+  common_progs = _dbus_getenv ("CommonProgramFiles");
+
   if (common_progs) 
     {
       if (!_dbus_string_append (&progs, common_progs)) 
@@ -3116,7 +3306,7 @@ test_default_session_servicedirs (void)
           _dbus_string_free (&progs);
           return FALSE;
         }
-      test_service_dir_matches[1] = _dbus_string_get_const_data(&progs);
+      test_session_service_dir_matches[1] = _dbus_string_get_const_data(&progs);
     }
 #endif
   dirs = NULL;
@@ -3160,7 +3350,7 @@ test_default_session_servicedirs (void)
   while ((link = _dbus_list_pop_first_link (&dirs)))
     {
       printf ("    test service dir: %s\n", (char *)link->data);
-      if (test_service_dir_matches[i] == NULL)
+      if (test_session_service_dir_matches[i] == NULL)
         {
           printf ("more directories parsed than in match set\n");
           dbus_free (link->data);
@@ -3169,12 +3359,12 @@ test_default_session_servicedirs (void)
           return FALSE;
         }
  
-      if (strcmp (test_service_dir_matches[i], 
+      if (strcmp (test_session_service_dir_matches[i], 
                   (char *)link->data) != 0)
         {
           printf ("%s directory does not match %s in the match set\n", 
                   (char *)link->data,
-                  test_service_dir_matches[i]);
+                  test_session_service_dir_matches[i]);
           dbus_free (link->data);
           _dbus_list_free_link (link);
           _dbus_string_free (&progs);
@@ -3187,10 +3377,10 @@ test_default_session_servicedirs (void)
       _dbus_list_free_link (link);
     }
   
-  if (test_service_dir_matches[i] != NULL)
+  if (test_session_service_dir_matches[i] != NULL)
     {
       printf ("extra data %s in the match set was not matched\n",
-              test_service_dir_matches[i]);
+              test_session_service_dir_matches[i]);
 
       _dbus_string_free (&progs);
       return FALSE;
@@ -3199,7 +3389,140 @@ test_default_session_servicedirs (void)
   _dbus_string_free (&progs);
   return TRUE;
 }
-			   
+
+static const char *test_system_service_dir_matches[] = 
+        {
+#ifdef DBUS_UNIX
+         "/testusr/testlocal/testshare/dbus-1/system-services",
+         "/testusr/testshare/dbus-1/system-services",
+#endif
+         DBUS_DATADIR"/dbus-1/system-services",
+#ifdef DBUS_UNIX
+         "/lib/dbus-1/system-services",
+#endif
+
+#ifdef DBUS_WIN
+         NULL,
+#endif
+         NULL
+        };
+
+static dbus_bool_t
+test_default_system_servicedirs (void)
+{
+  DBusList *dirs;
+  DBusList *link;
+  DBusString progs;
+#ifndef DBUS_UNIX
+  const char *common_progs;
+#endif
+  int i;
+
+  /* On Unix we don't actually use this variable, but it's easier to handle the
+   * deallocation if we always allocate it, whether needed or not */
+  if (!_dbus_string_init (&progs))
+    _dbus_assert_not_reached ("OOM allocating progs");
+
+#ifndef DBUS_UNIX
+  common_progs = _dbus_getenv ("CommonProgramFiles");
+
+  if (common_progs) 
+    {
+      if (!_dbus_string_append (&progs, common_progs)) 
+        {
+          _dbus_string_free (&progs);
+          return FALSE;
+        }
+
+      if (!_dbus_string_append (&progs, "/dbus-1/system-services")) 
+        {
+          _dbus_string_free (&progs);
+          return FALSE;
+        }
+      test_system_service_dir_matches[1] = _dbus_string_get_const_data(&progs);
+    }
+#endif
+  dirs = NULL;
+
+  printf ("Testing retrieving the default system service directories\n");
+  if (!_dbus_get_standard_system_servicedirs (&dirs))
+    _dbus_assert_not_reached ("couldn't get stardard dirs");
+
+  /* make sure our defaults end with share/dbus-1/system-service */
+  while ((link = _dbus_list_pop_first_link (&dirs)))
+    {
+      DBusString path;
+      
+      printf ("    default service dir: %s\n", (char *)link->data);
+      _dbus_string_init_const (&path, (char *)link->data);
+      if (!_dbus_string_ends_with_c_str (&path, "dbus-1/system-services"))
+        {
+          printf ("error with default system service directories\n");
+	      dbus_free (link->data);
+    	  _dbus_list_free_link (link);
+          _dbus_string_free (&progs);
+          return FALSE;
+        }
+ 
+      dbus_free (link->data);
+      _dbus_list_free_link (link);
+    }
+
+#ifdef DBUS_UNIX
+  if (!_dbus_setenv ("XDG_DATA_HOME", "/testhome/foo/.testlocal/testshare"))
+    _dbus_assert_not_reached ("couldn't setenv XDG_DATA_HOME");
+
+  if (!_dbus_setenv ("XDG_DATA_DIRS", ":/testusr/testlocal/testshare: :/testusr/testshare:"))
+    _dbus_assert_not_reached ("couldn't setenv XDG_DATA_DIRS");
+#endif
+  if (!_dbus_get_standard_system_servicedirs (&dirs))
+    _dbus_assert_not_reached ("couldn't get stardard dirs");
+
+  /* make sure we read and parse the env variable correctly */
+  i = 0;
+  while ((link = _dbus_list_pop_first_link (&dirs)))
+    {
+      printf ("    test service dir: %s\n", (char *)link->data);
+      if (test_system_service_dir_matches[i] == NULL)
+        {
+          printf ("more directories parsed than in match set\n");
+          dbus_free (link->data);
+          _dbus_list_free_link (link);
+          _dbus_string_free (&progs);
+          return FALSE;
+        }
+ 
+      if (strcmp (test_system_service_dir_matches[i], 
+                  (char *)link->data) != 0)
+        {
+          printf ("%s directory does not match %s in the match set\n", 
+                  (char *)link->data,
+                  test_system_service_dir_matches[i]);
+          dbus_free (link->data);
+          _dbus_list_free_link (link);
+          _dbus_string_free (&progs);
+          return FALSE;
+        }
+
+      ++i;
+
+      dbus_free (link->data);
+      _dbus_list_free_link (link);
+    }
+  
+  if (test_system_service_dir_matches[i] != NULL)
+    {
+      printf ("extra data %s in the match set was not matched\n",
+              test_system_service_dir_matches[i]);
+
+      _dbus_string_free (&progs);
+      return FALSE;
+    }
+    
+  _dbus_string_free (&progs);
+  return TRUE;
+}
+		   
 dbus_bool_t
 bus_config_parser_test (const DBusString *test_data_dir)
 {
@@ -3213,6 +3536,13 @@ bus_config_parser_test (const DBusString *test_data_dir)
   if (!test_default_session_servicedirs())
     return FALSE;
 
+#ifdef DBUS_WIN
+  printf("default system service dir skipped\n");
+#else
+  if (!test_default_system_servicedirs())
+    return FALSE;
+#endif
+
   if (!process_test_valid_subdir (test_data_dir, "valid-config-files", VALID))
     return FALSE;
 
