@@ -1,3 +1,4 @@
+#include <config.h>
 #include "test-utils.h"
 
 typedef struct
@@ -7,24 +8,13 @@ typedef struct
 
 } CData;
 
-static dbus_bool_t
-connection_watch_callback (DBusWatch     *watch,
-                           unsigned int   condition,
-                           void          *data)
-{
-  return dbus_watch_handle (watch, condition);
-}
-
 static dbus_bool_t
 add_watch (DBusWatch *watch,
 	   void      *data)
 {
   CData *cd = data;
 
-  return _dbus_loop_add_watch (cd->loop,
-                               watch,
-                               connection_watch_callback,
-                               cd, NULL);
+  return _dbus_loop_add_watch (cd->loop, watch);
 }
 
 static void
@@ -33,16 +23,7 @@ remove_watch (DBusWatch *watch,
 {
   CData *cd = data;
   
-  _dbus_loop_remove_watch (cd->loop,
-                           watch, connection_watch_callback, cd);  
-}
-
-static void
-connection_timeout_callback (DBusTimeout   *timeout,
-                             void          *data)
-{
-  /* Can return FALSE on OOM but we just let it fire again later */
-  dbus_timeout_handle (timeout);
+  _dbus_loop_remove_watch (cd->loop, watch);
 }
 
 static dbus_bool_t
@@ -51,8 +32,7 @@ add_timeout (DBusTimeout *timeout,
 {
   CData *cd = data;
 
-  return _dbus_loop_add_timeout (cd->loop,
-                                 timeout, connection_timeout_callback, cd, NULL);
+  return _dbus_loop_add_timeout (cd->loop, timeout);
 }
 
 static void
@@ -61,8 +41,7 @@ remove_timeout (DBusTimeout *timeout,
 {
   CData *cd = data;
 
-  _dbus_loop_remove_timeout (cd->loop,
-                             timeout, connection_timeout_callback, cd);
+  _dbus_loop_remove_timeout (cd->loop, timeout);
 }
 
 static void
@@ -224,28 +203,13 @@ serverdata_new (DBusLoop       *loop,
   return sd;
 }
 
-static dbus_bool_t
-server_watch_callback (DBusWatch     *watch,
-                       unsigned int   condition,
-                       void          *data)
-{
-  /* FIXME this can be done in dbus-mainloop.c
-   * if the code in activation.c for the babysitter
-   * watch handler is fixed.
-   */
-
-  return dbus_watch_handle (watch, condition);
-}
-
 static dbus_bool_t
 add_server_watch (DBusWatch  *watch,
                   void       *data)
 {
   ServerData *context = data;
 
-  return _dbus_loop_add_watch (context->loop,
-                               watch, server_watch_callback, context,
-                               NULL);
+  return _dbus_loop_add_watch (context->loop, watch);
 }
 
 static void
@@ -254,16 +218,7 @@ remove_server_watch (DBusWatch  *watch,
 {
   ServerData *context = data;
   
-  _dbus_loop_remove_watch (context->loop,
-                           watch, server_watch_callback, context);
-}
-
-static void
-server_timeout_callback (DBusTimeout   *timeout,
-                         void          *data)
-{
-  /* can return FALSE on OOM but we just let it fire again later */
-  dbus_timeout_handle (timeout);
+  _dbus_loop_remove_watch (context->loop, watch);
 }
 
 static dbus_bool_t
@@ -272,8 +227,7 @@ add_server_timeout (DBusTimeout *timeout,
 {
   ServerData *context = data;
 
-  return _dbus_loop_add_timeout (context->loop,
-                                 timeout, server_timeout_callback, context, NULL);
+  return _dbus_loop_add_timeout (context->loop, timeout);
 }
 
 static void
@@ -282,8 +236,7 @@ remove_server_timeout (DBusTimeout *timeout,
 {
   ServerData *context = data;
   
-  _dbus_loop_remove_timeout (context->loop,
-                             timeout, server_timeout_callback, context);
+  _dbus_loop_remove_timeout (context->loop, timeout);
 }
 
 dbus_bool_t
@@ -303,16 +256,20 @@ test_server_setup (DBusLoop      *loop,
                                         sd,
                                         serverdata_free))
     {
-      return FALSE;
+      goto nomem;
     }
 
+  sd = serverdata_new (loop, server);
+  if (sd == NULL)
+    goto nomem;
+
   if (!dbus_server_set_timeout_functions (server,
                                           add_server_timeout,
                                           remove_server_timeout,
                                           NULL,
                                           sd, serverdata_free))
     {
-      return FALSE;
+      goto nomem;
     }   
   return TRUE;
 
@@ -329,6 +286,8 @@ void
 test_server_shutdown (DBusLoop         *loop,
                       DBusServer       *server)
 {
+  dbus_server_disconnect (server);
+
   if (!dbus_server_set_watch_functions (server,
                                         NULL, NULL, NULL,
                                         NULL,
